﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.IO;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Web;

namespace Julklapp.Models
{
	public class LosAbfrageModel
	{
		[Required]
		[Display(Name = "Meine Emailadresse:")]
		public string Email { get; set; }
	}

	public class LosNameModel
	{
		public string BeschenktePerson { get; set; }
	}

	public interface IEmailService
	{
		bool Send(string emailAddress, string subject, string body);
	}

	public class EmailService : IEmailService
	{
		public bool Send(string emailAddress, string subject, string body)
		{
			throw new NotImplementedException();
		}
	}

	public class UserNameService
	{
		public static List<string> AllUserNames
		{
			get
			{
				return new List<string>
				       	{
							"Andrej",
							"Anton",
				       		"Dominika",
							"Mark",
							"Oliver"
				       	};
			}
		}

		public string GetGiftedPersonFor(string userName)
		{
			var fileName = Path.Combine(
				AppDomain.CurrentDomain.BaseDirectory, "App_Data", "match.txt");
			
			if (!File.Exists(fileName))
			{
				File.Create(fileName).Close();

				var people = new List<string>(AllUserNames);
				var gifted = new List<string>(AllUserNames);
				var indices = new int[people.Count];
				for (var i = 0; i < indices.Length; i++)
					indices[i] = i;

				while (indices.Any(i => people[i] == gifted[i]))
				{
					Shuffle(gifted);
				}

				File.WriteAllLines(fileName, indices.Select(i=>string.Format("{0}->{1}", Encode(people[i]), gifted[i])));
			}

			var lines = File.ReadAllLines(fileName);
			var match = lines.First(l => l.StartsWith(Encode(userName)));
			return match.Split(new[] {"->"}, StringSplitOptions.None)[1];
		}

		private static string Encode(string name)
		{
			var encoding = new ASCIIEncoding();
			var sha1 = SHA1.Create();
			var computedHash = sha1.ComputeHash(encoding.GetBytes(name));

			return BytesToHexString(computedHash);
		}

		// convert the byte array back to a true string
		private static string BytesToHexString(byte[] bytesInput)
		{
			var strTemp = new StringBuilder(bytesInput.Length * 2);
			foreach (var b in bytesInput)
			{
				strTemp.Append(b.ToString("X02"));
			}
			return strTemp.ToString();
		}

		/// <summary>
		/// Shuffles the items in the list, changes the list! Uses a Random instance with changing seed.
		/// </summary>
		/// <remarks>
		/// User Fisher-Yates (see http://en.wikipedia.org/wiki/Fisher-Yates_shuffle)
		/// recommended here: http://nunos.zi-yu.com/2008/03/shuffle-a-collection/
		/// </remarks>
		public static IList Shuffle(IList elements)
		{
			var random = new Random();

			int n = elements.Count;

			while (n > 1)
			{
				int k = random.Next(n);
				--n;
				var temp = elements[n];
				elements[n] = elements[k];
				elements[k] = temp;
			}

			return elements;
		}
	}
}